/*
 * A test of executing code in new tasks and threads.
 *
 * Copyright (C) 2022, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/sys/err.h>

#include <fsclient/process.h>
#include <systypes/format.h>

#include <stdio.h>
#include <stdlib.h>



int main(int argc, char *argv[])
{
  long err;

  if (argc < 3)
  {
    printf("Need a number and the program to run.\n");
    return 1;
  }

  int num_processes = atoi(argv[1]);

  for (int i = 1; i <= num_processes; i++)
  {
    printf("[%d/%d] Start process...\n", i, num_processes);

    /* Start a process for the given program. */

    process_t *process;

    err = process_spawn(argc - 2, (const char **) argv + 2, NULL, &process);

    if (err)
    {
      printf("Could not start process: %s\n", l4sys_errtostr(err));
      return 1;
    }

    printf("Finished program initiation.\n");

    /* Wait for a signal from the process. */

    notify_flags_t flags;
    notify_values_t values;

    err = process_wait(process, &flags, &values);

    if (err)
    {
      printf("Could not wait for process: %s\n", l4sys_errtostr(err));
      return 1;
    }

    printf("[%d/%d] End process (flags %" pFMTnotify_flags "x values: %ld, %ld)\n", i, num_processes, flags, values.sig, values.val);
  }

  printf("End of test.\n");
  return 0;
}

/* vim: tabstop=2 expandtab shiftwidth=2
*/
